import decimal
import requests
from dataclasses import asdict

from yandex_delivery.exceptions import ErrorHandler, APIError
from yandex_delivery.objects import RoutePointWithAddress, ClientRequirements, Claim


class YandexDeliveryClient:
    """
    Клиент для работы с API Яндекс Доставка
    """
    def __init__(self, api_key: str):
        self.api_key = api_key
        self.base_url = "https://b2b.taxi.yandex.net/b2b/cargo/integration/v2/"
        self._session = requests.Session()
        self._session.headers.update({
            "Authorization": f"Bearer {api_key}",
            "Accept-Language": "ru/ru",
            "Content-Type": "application/json"
        })
        self.timeout = 10

    def check_price(
            self,
            shop_route: RoutePointWithAddress,
            client_route: RoutePointWithAddress,
            client_requirements: ClientRequirements,
            skip_door_to_door: bool = False
    ) -> decimal.Decimal:
        """
        Предварительная оценка стоимости доставки без создания заявки.

        Args:
            shop_route: Точка маршрута с адресом магазина
            client_route: Точка маршрута с адресом клиента
            client_requirements: Требования клиента к доставке
            skip_door_to_door: Флаг пропуска опции доставки до двери (по умолчанию False)

        Returns:
            Decimal: Стоимость доставки

        Raises:
            APIError: Если API вернул ошибку с кодом и сообщением
            RequestException: При проблемах с сетевым запросом
            ValueError: При некорректных данных в ответе

        Examples:
            >>> price = client.check_price(
            ...     shop_route=shop_point,
            ...     client_route=client_point,
            ...     client_requirements=requirements
            ... )
            >>> print(f"Стоимость доставки: {price} руб.")
        """
        claim = Claim(
            items=None,
            route_points=[shop_route, client_route],
            requirements=client_requirements,
            skip_door_to_door=skip_door_to_door,
        )

        try:
            response = self._session.post(
                url=f"{self.base_url}check-price",
                json=asdict(claim),
                timeout=self.timeout
            )

            response_data = response.json()

            if response.status_code >= 400:
                ErrorHandler.handle_error_response(response.status_code, response_data)

            price = response_data.get('price')
            if price is None:
                raise ValueError("Price is missing in the API response")

            return decimal.Decimal(str(price))  # Безопасное преобразование в Decimal

        except requests.exceptions.RequestException as e:
            raise APIError(
                message=f"Network error occurred: {str(e)}",
                code="network_error",
                http_status=None
            ) from e

        except (ValueError, TypeError) as e:
            raise APIError(
                message=f"Failed to parse API response: {str(e)}",
                code="parse_error",
                http_status=response.status_code if 'response' in locals() else None
            ) from e
